@@ -14,6 +14,7 @@ import sys
 import platform
 import warnings
 from contextlib import contextmanager
+import subprocess
 
 import numpy as np
 from numpy import (
@@ -39,8 +40,8 @@ if sys.version_info[0] >= 3:
     long = int
 
 # Recommended minimum versions
-minimum_numpy_version = ('1', '7', '0')
-
+from distutils.version import LooseVersion
+minimum_numpy_version = LooseVersion('1.7.0')
 
 class test_numexpr(TestCase):
     """Testing with 1 thread"""
@@ -307,6 +308,36 @@ class test_numexpr(TestCase):
         res = evaluate('where(a, b, c)')
         assert_array_equal(res, c)
 
+    
+    def test_refcount(self):
+        # Regression test for issue #310
+        a = array([1])
+        assert sys.getrefcount(a) == 2
+        evaluate('1')
+        assert sys.getrefcount(a) == 2
+
+    def test_locals_clears_globals(self):
+        # Check for issue #313, whereby clearing f_locals also clear f_globals
+        # if in the top-frame. This cannot be done inside `unittest` as it is always 
+        # executing code in a child frame.
+        script = r';'.join([
+                r"import numexpr as ne",
+                r"a=10",
+                r"ne.evaluate('1')",
+                r"a += 1",
+                r"ne.evaluate('2', local_dict={})",
+                r"a += 1",
+                r"ne.evaluate('3', global_dict={})",
+                r"a += 1",
+                r"ne.evaluate('4', local_dict={}, global_dict={})",
+                r"a += 1",
+            ])
+        # Raises CalledProcessError on a non-normal exit
+        check = subprocess.check_call([sys.executable, '-c', script])
+        # Ideally this test should also be done against ipython but it's not 
+        # a requirement.
+
+
 
 class test_numexpr2(test_numexpr):
     """Testing with 2 threads"""
@@ -474,7 +505,7 @@ class test_evaluate(TestCase):
         # Execution order set here so as to not use too many threads
         # during the rest of the execution.  See #33 for details.
         def test_changing_nthreads_00_inc(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(1, 7):
                 numexpr.set_num_threads(nthreads)
@@ -482,7 +513,7 @@ class test_evaluate(TestCase):
                 assert_array_almost_equal(b, c)
 
         def test_changing_nthreads_01_dec(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(6, 1, -1):
                 numexpr.set_num_threads(nthreads)
@@ -636,8 +667,7 @@ def test_expressions():
                                                 neval, type(neval), shape(neval))
 
         method.description = ('test_expressions(%s, test_scalar=%r, '
-                              'dtype=%r, optimization=%r, exact=%r)') \
-                             % (expr, test_scalar, dtype.__name__, optimization, exact)
+                              'dtype=%r, optimization=%r, exact=%r)') % (expr, test_scalar, dtype.__name__, optimization, exact)
         test_no[0] += 1
         method.__name__ = 'test_scalar%d_%s_%s_%s_%04d' % (test_scalar,
                                                            dtype.__name__,
@@ -926,17 +956,39 @@ def _environment(key, value):
         else:
             del os.environ[key]
 
-
 # Test cases for the threading configuration
 class test_threading_config(TestCase):
+    def test_max_threads_unset(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "if 'NUMEXPR_MAX_THREADS' in os.environ: os.environ.pop('NUMEXPR_MAX_THREADS')",
+                "import numexpr",
+                "assert(numexpr.nthreads <= 8)",
+                "exit(0)"])
+        subprocess.check_call([sys.executable, '-c', script])
+
+    def test_max_threads_set(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "os.environ['NUMEXPR_MAX_THREADS'] = '4'",
+                "import numexpr",
+                "assert(numexpr.MAX_THREADS == 4)",
+                "exit(0)"])
+        subprocess.check_call([sys.executable, '-c', script])
+
     def test_numexpr_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
+            # NUMEXPR_NUM_THREADS has priority
             with _environment('NUMEXPR_NUM_THREADS', '3'):
-                self.assertEqual(3, numexpr.detect_number_of_threads())
+                self.assertEqual(3, numexpr._init_num_threads())
 
     def test_omp_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
-            self.assertEqual(5, numexpr.detect_number_of_threads())
+            self.assertEqual(5, numexpr._init_num_threads())
 
 
 # Case test for threads
@@ -1010,11 +1062,10 @@ def print_versions():
     from numexpr.cpuinfo import cpu
     import platform
 
-    np_version = tuple( ver for ver in np.__version__.split('.') )
+    np_version = LooseVersion(np.__version__)
 
-    if minimum_numpy_version < np_version:
-        print('*Warning*: NumPy version is lower than recommended: %s < %s' % \
-              (np.__version__, minimum_numpy_version))
+    if np_version < minimum_numpy_version:
+        print('*Warning*: NumPy version is lower than recommended: %s < %s' % (np_version, minimum_numpy_version))
     print('-=' * 38)
     print('Numexpr version:   %s' % numexpr.__version__)
     print('NumPy version:     %s' % np.__version__)
